function LCMAOEA(Global)
% <algorithm> <H-N> 
% Generating multiple reference vectors for a class of many-objective
% optimization problems with degenerate Pareto fronts
% Author: Yicun Hua, Yaochu Jin, Kuangrong Hao, Yuan Cao

%--------------------------------------------------------------------------
% The copyright of the PlatEMO belongs to the BIMK Group. You are free to
% use the PlatEMO for research purposes. All publications which use this
% platform or any code in the platform should acknowledge the use of
% "PlatEMO" and reference "Ye Tian, Ran Cheng, Xingyi Zhang, and Yaochu
% Jin, PlatEMO: A MATLAB Platform for Evolutionary Multi-Objective
% Optimization, 2016".
%--------------------------------------------------------------------------

% Copyright (c) 2016-2017 BIMK Group

     %% Generate the initial weight vectors
     [W,Global.N] = UniformPoint(Global.N,Global.M);
     T = ceil(Global.N/10);
     
     %% Detect the neighbours of each weight vector
     B = pdist2(W,W);
     [~,B] = sort(B,2);
     B = B(:,1:T);
     
     %% Generate random population
     Population = Global.Initialization();
     Z = min(Population.objs,[],1);
     luasV = [];
     sta = 0;
     h = 0;
     
     %% Optimization
     while Global.NotTermination(Population)
        
         if h ==0 % The effective vectors are unstable
             PopObj = Population.objs-repmat(Z,size(Population.objs,1),1);
             Angle = acos(1-pdist2(PopObj,W,'cosine'));
             [~,associate] = min(Angle,[],2);
             uasV = unique(associate); % Non repetitive effective vectors
             
             % Determine whether the effective vectors are stable
             if isequal(uasV,luasV)
                sta = sta+1;
                luasV = uasV;
             else
%                  sta = sta-1;
%                  if sta < 0
%                      sta = 0;
%                  end
                 sta = 0;
                 luasV = uasV;
             end
             
             if sta > 5 % The effective vectors are stable
                 
                 if (length(uasV) < Global.N/3)
                     % Find boundary vectors
                     [~,b] = max(W);
                     effV = unique([uasV;b']);
                     
                     % How many more vectors do we need?
                     lack = size(W,1) - length(effV);
                     
                     if lack == 0
                         W = W(effV,:);
                     else
                         % Map individuals to the hyperplane
                         Pobj = Population.objs;
                         Pm = [];
                         for mp = 1: length(uasV)
                             p = find(associate==uasV(mp));
                             for pi = 1:length(p)
                                 [ Pm(p(pi),:),flage ] = mapp( Pobj(p(pi),:), W(uasV(mp),:) );
                             end
                         end
                         
                         % The mapped individuals
                         uPm = unique(Pm,'rows');
                         
                         if size(uPm,1) > lack
                             % Clustering
                             CN = clusterdata(uPm,'maxclust',lack,'distance','cosine','linkage','ward');
                            
                             % Calculate cluster centers
                             for t = 1:lack
                                 current = find(CN == t);                                 
                                 cent = sum(uPm(current,:),1)/length(current);
                                 centre(t,:) = cent/sqrt(sum(cent.^2,2));
                             end
                             
                         elseif size(uPm,1)==lack
                             for c = 1:lack
                                 centre(c,:) = uPm(c,:)/sqrt(sum(uPm(c,:).^2,2)); 
                             end
                             
                         else                            
                             for c = 1:size(uPm,1)
                                 centre(c,:) = uPm(c,:)/sqrt(sum(uPm(c,:).^2,2));
                             end
                             % Add some random vectors within effective area                             % area
                             centre(size(uPm,1)+1:lack,:)  = rand(lack-size(uPm,1),size(uPm,2)).*repmat(max([Pobj;W(uasV,:)],[],1),lack-size(uPm,1),1);
                         end
                         
                         W = [W(effV,:);centre];
                         h = 1;
                     end
                 end
             end             
         else
         end
         
         % For each solution
         for i = 1 : Global.N
             % Choose the parents
             P = B(i,randperm(size(B,2)));
             
             % Generate an offspring
             Offspring = Global.Variation(Population(P(1:2)),1);
             
             % Update the ideal point
             Z = min(Z,Offspring.obj);
             
             % Update the neighbours and Environmental selection
             % PBI approach
             normW   = sqrt(sum(W(P,:).^2,2));
             normP   = sqrt(sum((Population(P).objs-repmat(Z,T,1)).^2,2));
             normO   = sqrt(sum((Offspring.obj-Z).^2,2));
             CosineP = sum((Population(P).objs-repmat(Z,T,1)).*W(P,:),2)./normW./normP;
             CosineO = sum(repmat(Offspring.obj-Z,T,1).*W(P,:),2)./normW./normO;
             g_old   = normP.*CosineP + 5*normP.*sqrt(1-CosineP.^2);
             g_new   = normO.*CosineO + 5*normO.*sqrt(1-CosineO.^2);
             Population(P(g_old>g_new)) = Offspring;
             
         end
     end
end